@@ -48,10 +48,9 @@ class Importer(BaseAction):
                     env_status = self.env.status()
                     reloaded = True
 
+                placement = self.deployment.get_unit_placement(svc, env_status)
                 for mid in range(cur_units, svc.num_units):
-                    mspec = self.deployment.get_unit_placement(
-                        svc, mid, env_status)
-                    self.env.add_unit(svc.name, mspec)
+                    self.env.add_unit(svc.name, placement.get(mid))
             else:
                 self.env.add_units(svc.name, abs(delta))
 
@@ -68,6 +67,7 @@ class Importer(BaseAction):
 
     def deploy_services(self):
         self.log.info("Deploying services...")
+        self.log.debug(self.env)
         env_status = self.env.status()
         reloaded = False
 
@@ -94,6 +94,8 @@ class Importer(BaseAction):
             else:
                 num_units = svc.num_units
 
+            placement = self.deployment.get_unit_placement(svc, env_status)
+
             self.env.deploy(
                 svc.name,
                 charm.charm_url,
@@ -101,7 +103,7 @@ class Importer(BaseAction):
                 svc.config,
                 svc.constraints,
                 num_units,
-                self.deployment.get_unit_placement(svc, 0, env_status))
+                placement.get(0))
 
             if svc.annotations:
                 self.log.debug(" Setting annotations")
