@@ -8,6 +8,7 @@ Deployment automation for juju.
 
 
 import argparse
+import errno
 import logging
 import os
 import sys
@@ -138,7 +139,13 @@ def run():
     log = logging.getLogger("deployer.cli")
     start_time = time.time()
 
-    env = select_runtime(options.juju_env, options)
+    try:
+        env = select_runtime(options.juju_env, options)
+    except OSError as e:
+        if e.errno != errno.ENOENT:
+            raise
+        log.error("No juju binary found, have you installed juju?")
+        sys.exit(1)
     log.debug('Using runtime %s', env.__class__.__name__)
 
     config = ConfigStack(options.configs or [])
