@@ -1,5 +1,7 @@
 from os.path import abspath, isabs, join, dirname
+
 import logging
+import os
 import tempfile
 import shutil
 import urllib
@@ -58,7 +60,9 @@ class ConfigStack(object):
             raise ErrorExit()
         deploy_data = self.data[key]
         deploy_data = self._resolve_inherited(deploy_data)
-        return Deployment(key, deploy_data, self.include_dirs)
+        return Deployment(
+            key, deploy_data, self.include_dirs,
+            repo_path=os.environ.get("JUJU_REPOSITORY", ""))
 
     def load(self):
         data = {}
