@@ -6,7 +6,8 @@ import os
 import yaml
 
 from .charm import Charm
-from .service import Service
+from .feedback import Feedback
+from .service import Service, ServiceUnitPlacement
 from .relation import Endpoint
 from .utils import path_join, yaml_dump, ErrorExit, resolve_include
 
@@ -56,58 +57,10 @@ class Deployment(object):
             return -1
         return cmp(svc_a.name, svc_b.name)
 
-    @staticmethod
-    def _format_placement(machine, container=None):
-        if container:
-            return "%s:%s" % (container, machine)
-        else:
-            return machine
-
-    def get_unit_placement(self, svc, unit_number, status):
-        unit_mapping = svc.unit_placement
-        if not unit_mapping:
-            return None
-        if len(unit_mapping) <= unit_number:
-            return None
-
-        unit_placement = placement = str(unit_mapping[unit_number])
-        container = None
-        u_idx = unit_number
-
-        if ':' in unit_placement:
-            container, placement = unit_placement.split(":")
-        if '=' in placement:
-            placement, u_idx = placement.split("=")
-
-        if placement.isdigit() and placement == "0":
-            return self._format_placement(placement, container)
-
-        with_service = status['services'].get(placement)
-        if with_service is None:
-            # Should be caught in validate relations but sanity check
-            # for concurrency.
-            self.log.error(
-                "Service %s to be deployed with non existant service %s",
-                svc.name, placement)
-            # Prefer continuing deployment with a new machine rather
-            # than an in-progress abort.
-            return None
-
-        svc_units = with_service['units']
-        if len(svc_units) <= unit_number:
-            self.log.warning(
-                "Service:%s deploy-with Service:%s, but no with unit found",
-                svc.name, placement)
-            return None
-        unit_names = svc_units.keys()
-        unit_names.sort()
-        machine = svc_units[unit_names[int(u_idx)]].get('machine')
-        if not machine:
-            self.log.warning(
-                "Service:%s deploy-with unit missing machine %s",
-                svc.name, unit_names[unit_number])
-            return None
-        return self._format_placement(machine, container)
+    def get_unit_placement(self, svc, status):
+        if isinstance(svc, (str, unicode)):
+            svc = self.get_service(svc)
+        return ServiceUnitPlacement(svc, self, status)
 
     def get_relations(self):
         if 'relations' not in self.data:
@@ -213,6 +166,7 @@ class Deployment(object):
         self.log.debug("Resolving configuration")
         # XXX TODO, rename resolve, validate relations
         # against defined services
+        feedback = Feedback()
         for svc_name, svc_data in self.data.get('services', {}).items():
             if not 'options' in svc_data:
                 continue
@@ -222,16 +176,21 @@ class Deployment(object):
 
             for k, v in svc_data['options'].items():
                 if not k in config:
-                    self.log.error(
-                        "Invalid config charm %s %s=%s", charm.name, k, v)
-                    raise ErrorExit()
+                    feedback.error(
+                        "Invalid config charm %s %s=%s" % (charm.name, k, v))
+                    continue
                 iv = self._resolve_include(svc_name, k, v)
+                if isinstance(iv, Feedback):
+                    feedback.extend(iv)
+                    continue
                 if iv is not None:
                     v = iv
                 options[k] = v
             svc_data['options'] = options
+        self._handle_feedback(feedback)
 
     def _resolve_include(self, svc_name, k, v):
+        feedback = Feedback()
         for include_type in ["file", "base64"]:
             if (not isinstance(v, basestring)
                 or not v.startswith(
@@ -240,70 +199,47 @@ class Deployment(object):
             include, fname = v.split("://", 1)
             ip = resolve_include(fname, self.include_dirs)
             if ip is None:
-                self.log.warning(
-                    "Invalid config %s.%s include not found %s",
-                    svc_name, k, v)
+                feedback.error(
+                    "Invalid config %s.%s include not found %s" % (
+                    svc_name, k, v))
                 continue
             with open(ip) as fh:
                 v = fh.read()
                 if include_type == "base64":
                     v = b64encode(v)
                 return v
+        if feedback:
+            return feedback
 
     def validate_relations(self):
         # Could extend to do interface matching against charms.
         services = dict([(s.name, s) for s in self.get_services()])
+        feedback = Feedback()
         for e_a, e_b in self.get_relations():
             for ep in [Endpoint(e_a), Endpoint(e_b)]:
                 if not ep.service in services:
-                    self.log.error(
+                    feedback.error(
                         ("Invalid relation in config,"
-                         " service %s not found, rel %s"),
-                        ep.service, "%s <-> %s" % (e_a, e_b))
-                    raise ErrorExit()
+                         " service %s not found, rel %s") % (
+                             ep.service, "%s <-> %s" % (e_a, e_b)))
+                    continue
+        self._handle_feedback(feedback)
 
     def validate_placement(self):
         services = dict([(s.name, s) for s in self.get_services()])
+        feedback = Feedback()
         for name, s in services.items():
-            unit_placement = s.unit_placement
-            if unit_placement is None:
-                continue
-            if not isinstance(unit_placement, list):
-                unit_placement = [unit_placement]
-            unit_placement = map(str, unit_placement)
-            for idx, p in enumerate(unit_placement):
-                if ':' in p:
-                    container, p = p.split(':')
-                    if container not in ('lxc', 'kvm'):
-                        self.log.error(
-                            "Invalid service:%s placement: %s",
-                            name, unit_placement[idx])
-                        raise ErrorExit()
-                if '=' in p:
-                    p, u_idx = p.split("=")
-                    if not u_idx.isdigit():
-                        self.log.error(
-                            "Invalid service:%s placement: %s",
-                            name, unit_placement[idx])
-                        raise ErrorExit()
-                if p.isdigit() and p == '0':
-                    continue
-                elif p.isdigit():
-                    self.log.error(
-                        "Service placement to machine not supported %s to %s",
-                        name, unit_placement[idx])
-                    raise ErrorExit()
-                elif p in services:
-                    if services[p].unit_placement:
-                        self.log.error(
-                            "Nested placement not supported %s -> %s -> %s" % (
-                                name, p, services[p].unit_placement))
-                        raise ErrorExit()
-                else:
-                    self.log.error(
-                        "Invalid service placement %s to %s" % (
-                            name, unit_placement[idx]))
-                    raise ErrorExit()
+            placement = self.get_unit_placement(s, {})
+            feedback.extend(placement.validate())
+        self._handle_feedback(feedback)
+
+    def _handle_feedback(self, feedback):
+        for e in feedback.get_errors():
+            self.log.error(e)
+        for w in feedback.get_warnings():
+            self.log.warning(w)
+        if feedback.has_errors:
+            raise ErrorExit()
 
     def save(self, path):
         with open(path, "w") as fh:
