@@ -62,7 +62,7 @@ class GoEnvironment(BaseEnvironment):
             return self.client.get_constraints(svc_name)
         except EnvError, e:
             if 'constraints do not apply to subordinate services' in str(e):
-                return None
+                return {}
             else:
                 raise e
 
