@@ -5,6 +5,7 @@ See <https://code.launchpad.net/~juju-gui/charms/precise/juju-gui/trunk>.
 """
 
 from .go import GoEnvironment
+from ..utils import parse_constraints
 
 
 class GUIEnvironment(GoEnvironment):
@@ -54,6 +55,8 @@ class GUIEnvironment(GoEnvironment):
         The repo argument is ignored but listed since the Importer always
         passes the value as a positional argument.
         """
+
+        constraints = parse_constraints(constraints)
         self.client.deploy(
             name, charm_url, config=config, constraints=constraints,
             num_units=num_units, machine_spec=force_machine)
