@@ -7,6 +7,10 @@ import StringIO
 import tempfile
 
 import deployer
+from deployer.config import ConfigStack
+
+
+TEST_OFFLINE = ("DEB_BUILD_ARCH" in os.environ or "TEST_OFFLINE" in os.environ)
 
 
 class Base(unittest.TestCase):
@@ -14,6 +18,13 @@ class Base(unittest.TestCase):
     test_data_dir = os.path.join(
         os.path.dirname(inspect.getabsfile(deployer)), "tests", "test_data")
 
+    def get_named_deployment(self, file_name, stack_name):
+        """ Get deployment from test_data file.
+        """
+        return ConfigStack(
+            [os.path.join(
+                self.test_data_dir, file_name)]).get(stack_name)
+
     def capture_logging(self, name="", level=logging.INFO,
                         log_file=None, formatter=None):
         if log_file is None:
