@@ -1,13 +1,14 @@
 import base64
 import StringIO
 import os
+import unittest
 
 
 from deployer.config import ConfigStack
 from deployer.deployment import Deployment
 from deployer.utils import setup_logging, ErrorExit
 
-from .base import Base
+from .base import Base, TEST_OFFLINE
 
 
 class DeploymentTest(Base):
@@ -21,6 +22,8 @@ class DeploymentTest(Base):
             [os.path.join(
                 self.test_data_dir, file_name)]).get(stack_name)
 
+    @unittest.skipIf(TEST_OFFLINE,
+                     "Requires configured bzr launchpad id and network access")
     def test_deployer(self):
         d = ConfigStack(
             [os.path.join(
@@ -41,7 +44,7 @@ class DeploymentTest(Base):
 
         # Load up overrides and resolves
         d.load_overrides(["key=abc"])
-        d.resolve_config()
+        d.resolve()
 
         # Verify include-base64
         self.assertEqual(d.get_service('newrelic').config, {'key': 'abc'})
@@ -97,24 +100,24 @@ class DeploymentTest(Base):
                         'nova-compute/2': {'machine': '1'},
                         'nova-compute/3': {'machine': '2'},
                         'nova-compute/4': {'machine': '3'}}}}}
-        svc = d.get_service('ceph')
-        self.assertEqual(d.get_unit_placement(svc, 0, status), '1')
-        self.assertEqual(d.get_unit_placement(svc, 1, status), '2')
-        self.assertEqual(d.get_unit_placement(svc, 2, status), None)
+        placement = d.get_unit_placement('ceph', status)
+        self.assertEqual(placement.get(0), '1')
+        self.assertEqual(placement.get(1), '2')
+        self.assertEqual(placement.get(2), None)
 
-        svc = d.get_service('quantum')
-        self.assertEqual(d.get_unit_placement(svc, 0, status), 'lxc:1')
-        self.assertEqual(d.get_unit_placement(svc, 2, status), 'lxc:3')
-        self.assertEqual(d.get_unit_placement(svc, 3, status), None)
+        placement = d.get_unit_placement('quantum', status)
+        self.assertEqual(placement.get(0), 'lxc:1')
+        self.assertEqual(placement.get(2), 'lxc:3')
+        self.assertEqual(placement.get(3), None)
 
-        svc = d.get_service('verity')
-        self.assertEqual(d.get_unit_placement(svc, 0, status), 'lxc:3')
+        placement = d.get_unit_placement('verity', status)
+        self.assertEqual(placement.get(0), 'lxc:3')
 
-        svc = d.get_service('mysql')
-        self.assertEqual(d.get_unit_placement(svc, 0, status), '0')
+        placement = d.get_unit_placement('mysql', status)
+        self.assertEqual(placement.get(0), '0')
 
-        svc = d.get_service('semper')
-        self.assertEqual(d.get_unit_placement(svc, 0, status), '3')
+        placement = d.get_unit_placement('semper', status)
+        self.assertEqual(placement.get(0), '3')
 
     def test_multiple_relations_no_weight(self):
         data = {"relations": {"wordpress": {"consumes": ["mysql"]},
