@@ -38,14 +38,17 @@ class TestGUIEnvironment(unittest.TestCase):
 
     def test_deploy(self, mock_client):
         # The environment uses the API to deploy charms.
+        # Constraints are converted to numeric values before calling the
+        # client deploy.
         self.env.connect()
         config = {'foo': 'bar'}
-        constraints = {'cpu': 4}
+        constraints = {'cpu-cores': '4', 'mem': '5G'}
         # Deploy a service: the repo argument is ignored.
         self.env.deploy(
             'myservice', 'cs:precise/service-42', repo='/my/repo/',
             config=config, constraints=constraints, num_units=2,
             force_machine=1)
+        expected = {'cpu-cores': 4, 'mem': 5 * 1024}
         mock_client().deploy.assert_called_once_with(
             'myservice', 'cs:precise/service-42', config=config,
-            constraints=constraints, num_units=2, machine_spec=1)
+            constraints=expected, num_units=2, machine_spec=1)
