@@ -14,7 +14,51 @@ from deployer import (
     cli,
     guiserver,
 )
-from deployer.deployment import Deployment
+from deployer.feedback import Feedback
+from deployer.tests.base import TEST_OFFLINE
+
+
+class TestDeploymentError(unittest.TestCase):
+
+    def test_error(self):
+        # A single error is properly stored and represented.
+        exception = guiserver.DeploymentError(['bad wolf'])
+        self.assertEqual(['bad wolf'], exception.errors)
+        self.assertEqual('bad wolf', str(exception))
+
+    def test_multiple_errors(self):
+        # Multiple deployment errors are properly stored and represented.
+        errors = ['error 1', 'error 2']
+        exception = guiserver.DeploymentError(errors)
+        self.assertEqual(errors, exception.errors)
+        self.assertEqual('error 1\nerror 2', str(exception))
+
+
+class TestGUIDeployment(unittest.TestCase):
+
+    def setUp(self):
+        # Set up a GUIDeployment instance and a Feedback object.
+        self.deployment = guiserver.GUIDeployment('mybundle', 'mydata')
+        self.feedback = Feedback()
+
+    def test_valid_deployment(self):
+        # If the bundle is well formed, the deployment proceeds normally.
+        self.assertIsNone(self.deployment._handle_feedback(self.feedback))
+
+    def test_warnings(self):
+        # Warning messages are properly logged.
+        self.feedback.warn('we are the Borg')
+        with mock.patch.object(self.deployment, 'log') as mock_log:
+            self.deployment._handle_feedback(self.feedback)
+        mock_log.warning.assert_called_once_with('we are the Borg')
+
+    def test_errors(self):
+        # A DeploymentError is raised if errors are found in the bundle.
+        self.feedback.error('error 1')
+        self.feedback.error('error 2')
+        with self.assertRaises(guiserver.DeploymentError) as cm:
+            self.deployment._handle_feedback(self.feedback)
+        self.assertEqual(['error 1', 'error 2'], cm.exception.errors)
 
 
 class DeployerFunctionsTestMixin(object):
@@ -40,6 +84,7 @@ class DeployerFunctionsTestMixin(object):
           num_units: 2
           constraints:
             arch: i386
+            mem: 4G
             cpu-cores: 4
           annotations:
             "gui-x": "494.347"
@@ -83,6 +128,8 @@ class DeployerFunctionsTestMixin(object):
         mock_env_instance.close.assert_called_once_with()
 
 
+@unittest.skipIf(TEST_OFFLINE,
+                 "Requires configured bzr launchpad id and network access")
 @mock.patch('deployer.guiserver.GUIEnvironment')
 class TestValidate(DeployerFunctionsTestMixin, unittest.TestCase):
 
@@ -99,6 +146,8 @@ class TestValidate(DeployerFunctionsTestMixin, unittest.TestCase):
             guiserver.validate(self.apiurl, self.password, self.bundle)
 
 
+@unittest.skipIf(TEST_OFFLINE,
+                 "Requires configured bzr launchpad id and network access")
 @mock.patch('deployer.guiserver.GUIEnvironment')
 class TestImportBundle(DeployerFunctionsTestMixin, unittest.TestCase):
 
@@ -142,7 +191,7 @@ class TestImportBundle(DeployerFunctionsTestMixin, unittest.TestCase):
         # The first argument passed to the importer is the environment.
         self.assertIs(mock_environment(), env)
         # The second argument is the deployment object.
-        self.assertIsInstance(deployment, Deployment)
+        self.assertIsInstance(deployment, guiserver.GUIDeployment)
         self.assertEqual(self.name, deployment.name)
         self.assertEqual(self.bundle, deployment.data)
         # The third and last argument is the options object.
@@ -185,7 +234,7 @@ class TestImportBundle(DeployerFunctionsTestMixin, unittest.TestCase):
             mock.call.status(),
             mock.call.deploy(
                 'mysql', 'cs:precise/mysql-28', '', None,
-                {'arch': 'i386', 'cpu-cores': 4}, 2, None),
+                {'arch': 'i386', 'cpu-cores': 4, 'mem': '4G'}, 2, None),
             mock.call.set_annotation(
                 'mysql', {'gui-y': '164.547', 'gui-x': '494.347'}),
             mock.call.deploy(
@@ -200,6 +249,32 @@ class TestImportBundle(DeployerFunctionsTestMixin, unittest.TestCase):
             mock.call.close(),
         ], any_order=True)
 
+    def test_deployment_errors(self, mock_environment):
+        # A DeploymentError is raised if the deployment fails.
+        bundle = {
+            'services': {
+                'wordpress': {
+                    'charm': 'cs:precise/wordpress-20',
+                    'options': {'no-such-option': 42},  # Invalid options.
+                },
+                'mysql': {
+                    'charm': 'cs:precise/mysql-28',
+                    'options': {'bad': 'wolf'},  # Invalid options.
+                },
+            },
+        }
+        self.addCleanup(self.cleanup_series_path)
+        with self.patch_juju_home():
+            with self.assertRaises(guiserver.DeploymentError) as cm:
+                guiserver.import_bundle(
+                    self.apiurl, self.password, self.name, bundle,
+                    self.options)
+        expected_errors = set([
+            'Invalid config charm cs:precise/wordpress-20 no-such-option=42',
+            'Invalid config charm cs:precise/mysql-28 bad=wolf',
+        ])
+        self.assertEqual(expected_errors, set(cm.exception.errors))
+
     def test_options(self, mock_environment):
         # Ensure the default options are sane for us.
         expected = argparse.Namespace(
