@@ -142,27 +142,73 @@ def extract_zip(zip_path, dir_path):
         os.chmod(extract_path, mode)
 
 
+UNITS_DICT = {
+    'M': 1,
+    'G': 1024,
+    'T': 1024 * 1024,
+    'P': 1024 * 1024 * 1024,
+}
+
+
+def _to_number(value):
+    """Convert a string to a numeric.
+
+    The returned value is either an int or a float, depending on the input.
+    Raises a ValueError if the value cannot be parsed.
+    """
+    try:
+        num = int(value)
+    except ValueError:
+        num = float(value)
+    return num
+
+
+def _convert_units_specifier(value):
+    """Convert a string that may have a unit specifier.
+
+    Given a string possibly containing a unit specifier, return the
+    the string representing the number and the units multiplier.
+    """
+    units = UNITS_DICT.get(value[-1], None)
+    if units is not None:
+        value = value[:-1]
+    else:
+        units = 1
+    return value, units
+
+
 def parse_constraints(value):
+    """Parse the constraints, converting size specifiers into ints.
+
+    Specifiers of 'M' and 'G' are supported.  The resulting value is the number
+    of megabytes.
+
+    The input is either a string of the form "k1=v1 ... kn=vn" or a dictionary.
+    """
+    if value is None:
+        return value
+    constraints_with_units = ['mem', 'root-disk']
+    numerics = ['cpu-cores', 'cpu-power'] + constraints_with_units
     constraints = {}
-    pairs = value.strip().split()
-    units_dict = {
-        'M': 1,
-        'G': 1024,
-    }
-    for p in pairs:
-        k, v = p.split('=')
-        units = units_dict.get(v[-1], None)
-        if units:
-            v = v[0:-1]
-        else:
+    if isinstance(value, dict):
+        constraints.update(value)
+    else:
+        pairs = value.strip().split()
+        for item in pairs:
+            k, v = item.split('=')
+            constraints[k] = v
+
+    for k, v in constraints.items():
+        if k in numerics:
             units = 1
-        try:
-            v = int(v) * units
-        except ValueError:
+            if k in constraints_with_units:
+                v, units = _convert_units_specifier(v)
             try:
-                v = float(v) * units
-            except:
-                pass
+                v = _to_number(v) * units
+            except ValueError:
+                raise ValueError(
+                    'Constraint {} has invalid value {}'.format(
+                        k, constraints[k]))
         constraints[k] = v
     return constraints
 
